package org.oscy.tapestry.breadcrumbs.annotation;

import java.lang.annotation.Documented;
import java.lang.annotation.ElementType;
import java.lang.annotation.Retention;
import java.lang.annotation.RetentionPolicy;
import java.lang.annotation.Target;

import org.apache.tapestry5.ioc.annotations.AnnotationUseContext;
import org.apache.tapestry5.ioc.annotations.UseWith;


/**
 * A marker annotation that controls breadcrumb behaviour on the attached page
 *
 */
@Target(ElementType.TYPE)
@Retention(RetentionPolicy.RUNTIME)
@Documented
@UseWith(AnnotationUseContext.PAGE)
public @interface BreadCrumb
{
	/**
	 * Whether to ignore the attached page and not include it in the breadcrumbs trail.
	 */
	boolean ignore() default false;

	
	/**
	 * Whether to reset the whole breadcrumbs list.
	 * <p/>
	 * If used in association with the {@link BreadCrumb#ignore()} property then it will first 
	 * reset the list and then ignore the page
	 */
	boolean reset() default false;
}