package org.oscy.tapestry.breadcrumbs.components;

import org.apache.tapestry5.BindingConstants;
import org.apache.tapestry5.Block;
import org.apache.tapestry5.ComponentEventCallback;
import org.apache.tapestry5.EventContext;
import org.apache.tapestry5.annotations.Parameter;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.internal.util.Holder;
import org.apache.tapestry5.ioc.Messages;
import org.apache.tapestry5.ioc.annotations.Inject;
import org.apache.tapestry5.ioc.annotations.Symbol;
import org.apache.tapestry5.runtime.Component;
import org.apache.tapestry5.services.ComponentSource;
import org.oscy.tapestry.breadcrumbs.other.BreadCrumbElement;
import org.oscy.tapestry.breadcrumbs.other.BreadCrumbsConstants;
import org.oscy.tapestry.breadcrumbs.other.BreadCrumbsEventConstants;

/**
 * Part of BreadCrumbsTrail that renders the markup inside a single data  &lt;li&gt;.
 * 
 * @tapestrydoc
 */
public class BreadCrumbInfo
{
	@Inject
	private ComponentSource componentSource;


    /**
     * The current breadcrumb being rendered.
     */
	@Property
	@Parameter(required = true, principal = true, autoconnect = true)
	private BreadCrumbElement value;

	
    /**
     * A Block to render before each breadcrumb. The default is null. This parameter is used to change the default, possibly including components to
     * allow the user to create new objects.
     */
	@Parameter(value = "block:before", defaultPrefix = BindingConstants.LITERAL)
	private Block before;

	
    /**
     * A Block to render after each breadcrumb. The default is null. This parameter is used to change the default, possibly including components to
     * allow the user to create new objects.
     */
	@Parameter(value = "block:after", defaultPrefix = BindingConstants.LITERAL)
	private Block after;

	
    /**
     * A Block to render each breadcrumb. The default is @{link org.apache.tapestry5.corelib.components.PageLink} along with the page title.
     * <p>
     * The page title is calculated by:
     * <ul>
     * <li>Using a page event {@link org.oscy.tapestry.breadcrumbs.other.BreadCrumbsEventConstants#GET_TITLE}</li>
     * <li>Using message catalogues by appending {@link org.oscy.tapestry.breadcrumbs.other.BreadCrumbsConstants#BREADCRUMBS_TITLE_SUFFIX} </li>
     * <li>Logical page name {@link org.oscy.tapestry.breadcrumbs.other.BreadCrumbElement#getLogicalPageName()}</li>
     * </ul>
     * </p>
     * The user can override this block in order to change default value, possibly including components to
     * allow the user to create new objects.
     */
	@SuppressWarnings("unused")
	@Property
	@Parameter(value = "block:inside", defaultPrefix = BindingConstants.LITERAL)
	private Block inside;

	@Inject
	@Symbol(BreadCrumbsConstants.BREADCRUMBS_TITLE_SUFFIX)
	private String titleSuffix;

	Object beginRender() {
		return before;
	}

	Object afterRender() {
		return after;
	}

	public String getNameTitle() {
		String pageName = value.getLogicalPageName();
		Component page = componentSource.getPage(pageName);
		String title = null;
		final Holder<String> valueHolder = Holder.create();
		ComponentEventCallback<String> callback = new ComponentEventCallback<String>()
		{
			public boolean handleResult(String result) {
				valueHolder.put(result);

				return true;
			}
		};

		boolean handled = page.getComponentResources().triggerContextEvent(BreadCrumbsEventConstants.GET_TITLE, value.getActivationContext(), callback);
		if(handled) {
			return valueHolder.get();
		}

		if (title == null) {
			String key = new StringBuilder(pageName).append(titleSuffix).toString();
			Messages messages = page.getComponentResources().getMessages();
			if (messages.contains(key)) {
				title = messages.get(key);
			}
		}

		return title != null ? title : pageName;
	}

	public String getPage() {
		return componentSource.getPage(value.getLogicalPageName()).getComponentResources().getCompleteId();
	}

	public Object[] getActivationContext() {
		EventContext context = value.getActivationContext();
		int count = context.getCount();

		Object[] pageActivationContext = new Object[count];

		for (int i = 0; i < count; i++)
			pageActivationContext[i] = context.get(Object.class, i);

		return pageActivationContext;
	}
}
