package org.oscy.tapestry.breadcrumbs.components;

import java.util.List;

import org.apache.tapestry5.annotations.Component;
import org.apache.tapestry5.annotations.Parameter;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.internal.TapestryInternalUtils;
import org.apache.tapestry5.ioc.annotations.Inject;
import org.apache.tapestry5.ioc.internal.util.CollectionFactory;
import org.apache.tapestry5.services.ComponentSource;
import org.oscy.tapestry.breadcrumbs.other.BreadCrumbElement;
import org.oscy.tapestry.breadcrumbs.other.BreadCrumbsModel;


/**
 * Part of {@link BreadCrumbsTrail} that renders out a series of crumbs within the  &lt;ul&gt;.
 * 
 * @tapestrydoc
 */
@SuppressWarnings("unused")
public class BreadCrumbsLoop
{
	
	private static final String BREADCRUMBS_FIRST_CLASS = "z-first";
	private static final String BREADCRUMBS_LAST_CLASS = "z-last";
	private static final String BREADCRUMBS_ACTIVE_CLASS = "z-active";

    /**
     * Parameter used to set the CSS class for each breadcrumb (each &lt;li&gt; element) within the &lt;ul&gt;. This is not
     * cached, so it will be recomputed for each breadcrumb.
     */
	@Parameter(cache = false)
	private String crumbClass;

	
    /**
     * Optional output parameter (only set during rendering) that identifies the current breadcrumb index. This is the index on
     * the page (i.e., always numbered from zero).
     */
	@Parameter
	private int index;

	
    /**
     * Object that provides access to the crumb and data models used to render the BreadCrumbTrail.
     */
	@Parameter(value = "componentResources.container")
	private BreadCrumbsModel breadCrumbsModel;

	
    /**
     * The current crumb being rendered, this is primarily an output parameter used to allow the BreadCrumbTrail, and the BreadCrumbTrail's
     * container, to know what object is being rendered.
     */
	@Parameter(required = true)
	@Property(write = false)
	private BreadCrumbElement value;

	@Inject
	private ComponentSource componentSource;

	@Component(parameters = "value=value", publishParameters = "before,after,inside")
	private BreadCrumbInfo breadCrumb;

	private int endCrumb;


	public String getCrumbClass() {
		List<String> classes = CollectionFactory.newList();

		String rc = crumbClass;

		if (rc != null)
			classes.add(rc);

		if (index == 0)
			classes.add(BREADCRUMBS_FIRST_CLASS);

		if (index == endCrumb)
			classes.add(BREADCRUMBS_LAST_CLASS);

		if (componentSource.getActivePage().getComponentResources().getCompleteId().equals(componentSource.getPage(value.getLogicalPageName()).getComponentResources().getCompleteId())) {
			classes.add(BREADCRUMBS_ACTIVE_CLASS);
		}

		return TapestryInternalUtils.toClassAttributeValue(classes);
	}

	void setupForCrumb(int index) {
		value = breadCrumbsModel.getCrumb(index);
	}

	boolean setupRender() {
		int size = breadCrumbsModel.getSize();
		endCrumb = size - 1;
		index = 0;

		return size != 0;
	}

	boolean beginRender() {
		// Setup for this crumb.

		setupForCrumb(index);

		return value != null;
	}

	boolean afterRender() {
		index++;

		// Abort the loop when we hit a null crumb, or when we've exhausted the
		// range we need to
		// display.

		return value == null || index > endCrumb;
	}

}
