package org.oscy.tapestry.breadcrumbs.components;

import java.util.List;

import org.apache.tapestry5.BindingConstants;
import org.apache.tapestry5.annotations.Component;
import org.apache.tapestry5.annotations.Parameter;
import org.apache.tapestry5.annotations.Property;
import org.apache.tapestry5.annotations.SessionState;
import org.apache.tapestry5.annotations.SupportsInformalParameters;
import org.apache.tapestry5.corelib.components.Any;
import org.oscy.tapestry.breadcrumbs.other.BreadCrumbElement;
import org.oscy.tapestry.breadcrumbs.other.BreadCrumbsConstants;
import org.oscy.tapestry.breadcrumbs.other.BreadCrumbsModel;
import org.oscy.tapestry.breadcrumbs.session.BreadCrumbsList;

/**
 * A BreadCrumbsTrail presents breadcrumbs data. It is a composite component, created in terms of several sub-components. The
 * sub-components are statically wired to the BreadCrumbsTrail, as it provides access to the data and other models that they need.
 *
 * @tapestrydoc
 * @see BreadCrumbsModel
 */
@SuppressWarnings("unused")
@SupportsInformalParameters
public class BreadCrumbsTrail implements BreadCrumbsModel
{

	@SessionState
	private BreadCrumbsList breadCrumbs;

    /**
     * CSS class for the &lt;ul&gt; element. In addition, informal parameters to the Grid are rendered in the &lt;ul&gt;
     * element.
     */
	@Parameter(name = "class", defaultPrefix = BindingConstants.LITERAL, value = BindingConstants.SYMBOL + ":" + BreadCrumbsConstants.BREADCRUMBS_TRAIL_CSS_CLASS)
	@Property(write = false)
	private String trailClass;

	@Component(parameters = "class=trailClass", inheritInformalParameters = true)
	@Property
	private Any trail;

	@Component(parameters = "value=value", publishParameters = "index,crumbClass,before,after,inside")
	@Property
	private BreadCrumbsLoop trailCrumbs;

    /**
     * Used to store the current object being rendered (for the current breadcrumb). This is used when parameter blocks are
     * provided to override the default rendering of a breadcrumb. the components within the block can
     * use the property bound to the crumb parameter to know what they should render.
     */
	@Parameter(principal = true)
	@Property
	private BreadCrumbElement value;


    /**
     * A wrapper around the provided List<BreadCrumbElement> that caches access to the availableCrumbs property.
     */
	private CachingBreadCrumbSource cachingSource;

	boolean setupRender() {
		cachingSource = new CachingBreadCrumbSource(breadCrumbs.getBreadCrumbs());

		return cachingSource.getAvailableCrumbs() == 0 ? false : true;
	}

	static class CachingBreadCrumbSource
	{
		private final List<BreadCrumbElement> delegate;

		private boolean availableCrumbsCached;

		private int availableCrumbs;

		CachingBreadCrumbSource(List<BreadCrumbElement> delegate) {
			this.delegate = delegate;
		}

		public int getAvailableCrumbs() {
			if (!availableCrumbsCached) {
				availableCrumbs = delegate.size();
				availableCrumbsCached = true;
			}

			return availableCrumbs;
		}

		public BreadCrumbElement getCrumbValue(int index) {
			return delegate.get(index);
		}
	}

	public int getSize() {
		return cachingSource.getAvailableCrumbs();
	}

	public BreadCrumbElement getCrumb(int crumbIndex) {
		return cachingSource.getCrumbValue(crumbIndex);
	}
}
