package org.oscy.tapestry.breadcrumbs.other;

import java.io.Serializable;

import org.apache.tapestry5.EventContext;
import org.apache.tapestry5.internal.TapestryInternalUtils;


/**
 * Object that holds information for one specific breadcrumb
 */
public class BreadCrumbElement implements Serializable
{
	private static final long serialVersionUID = -6160551440249774901L;
	private final String logicalPageName;
	private final EventContext activationContext;

    /**
     * Constructs a new {@link BreadCrumbElement}.
     * 
     * @param logicalPageName The name of the page      
     * @param activationContext The activation context 
     */
	public BreadCrumbElement(String logicalPageName, EventContext activationContext) {
		this.logicalPageName = logicalPageName;
		this.activationContext = activationContext;
	}
	
	/**
	 * @return Logical name of the page
	 * 
	 * @see org.apache.tapestry5.services.PageRenderRequestParameters#getLogicalPageName()
	 */

	public String getLogicalPageName() {
		return logicalPageName;
	}

	
	/**
	 * @return {@link org.apache.tapestry5.EventContext} of the page
	 * 
	 * @see org.apache.tapestry5.services.PageRenderRequestParameters#getActivationContext()
	 */
	public EventContext getActivationContext() {
		return activationContext;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((activationContext == null) ? 0 : activationContext.hashCode());
		result = prime * result + ((logicalPageName == null) ? 0 : logicalPageName.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;

		if (obj == null || getClass() != obj.getClass())
			return false;

		BreadCrumbElement other = (BreadCrumbElement) obj;
		return TapestryInternalUtils.isEqual(activationContext, other.getActivationContext()) && other.getLogicalPageName().equals(this.logicalPageName);
	}

}
