package org.oscy.tapestry.breadcrumbs.other;

import org.oscy.tapestry.breadcrumbs.annotation.BreadCrumb;
import org.oscy.tapestry.breadcrumbs.utils.AntPathMatcher;


/**
 * Object that holds rule information for a specific breadcrumb when configuring through contribution .Alternative to 
 * using {@link BreadCrumb}
 */
public class BreadCrumbRule
{
	private final String pattern;
	private final BreadCrumbAttribute attribute;

	
    /**
     * Constructs a new {@link BreadCrumbRule}.
     * 
     * @param pattern Path pattern for page rule matching.Mathing will be done using {@link AntPathMatcher}   
     * @param attribute The attribute for this rule
     */
	public BreadCrumbRule(String pattern, BreadCrumbAttribute attribute) {
		this.pattern = pattern;
		this.attribute = attribute;
	}

	
	/**
	 * 
	 * @return Path pattern
	 */
	public String getPattern() {
		return pattern;
	}

	
	/**
	 * 
	 * @return The attribute for this rule
	 */
	public BreadCrumbAttribute getAttribute() {
		return attribute;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((attribute == null) ? 0 : attribute.hashCode());
		result = prime * result + ((pattern == null) ? 0 : pattern.hashCode());
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		BreadCrumbRule other = (BreadCrumbRule) obj;
		if (attribute != other.attribute)
			return false;
		if (pattern == null) {
			if (other.pattern != null)
				return false;
		} else if (!pattern.equals(other.pattern))
			return false;
		return true;
	}

}
