package org.oscy.tapestry.breadcrumbs.services;

import java.io.IOException;

import org.apache.tapestry5.Asset;
import org.apache.tapestry5.MarkupWriter;
import org.apache.tapestry5.annotations.Path;
import org.apache.tapestry5.internal.services.DocumentLinker;
import org.apache.tapestry5.ioc.Configuration;
import org.apache.tapestry5.ioc.MappedConfiguration;
import org.apache.tapestry5.ioc.OrderedConfiguration;
import org.apache.tapestry5.ioc.ServiceBinder;
import org.apache.tapestry5.ioc.annotations.Symbol;
import org.apache.tapestry5.services.ApplicationStateContribution;
import org.apache.tapestry5.services.ApplicationStateCreator;
import org.apache.tapestry5.services.Environment;
import org.apache.tapestry5.services.LibraryMapping;
import org.apache.tapestry5.services.MarkupRenderer;
import org.apache.tapestry5.services.MarkupRendererFilter;
import org.apache.tapestry5.services.PageRenderRequestFilter;
import org.apache.tapestry5.services.PageRenderRequestHandler;
import org.apache.tapestry5.services.PageRenderRequestParameters;
import org.apache.tapestry5.services.javascript.StylesheetLink;
import org.oscy.tapestry.breadcrumbs.internal.services.BreadCrumbsService;
import org.oscy.tapestry.breadcrumbs.internal.services.impl.BreadCrumbsServiceImpl;
import org.oscy.tapestry.breadcrumbs.other.BreadCrumbsConstants;
import org.oscy.tapestry.breadcrumbs.session.BreadCrumbsList;
import org.oscy.tapestry.breadcrumbs.session.impl.BreadCrumbsListImpl;

public class BreadCrumbsModule
{

	public static void bind(final ServiceBinder binder) {
		binder.bind(BreadCrumbsService.class, BreadCrumbsServiceImpl.class);
	}

	public static void contributeFactoryDefaults(final MappedConfiguration<String, String> configuration) {
		configuration.add(BreadCrumbsConstants.BREADCRUMBS_MAX_CRUMBS_TO_SAVE, "10");
		configuration.add(BreadCrumbsConstants.BREADCRUMBS_DISCARD_DUPLICATES, "true");
		configuration.add(BreadCrumbsConstants.BREADCRUMBS_TRAIL_CSS_CLASS, "t-zbreadcrumbs");
		configuration.add(BreadCrumbsConstants.BREADCRUMBS_TITLE_SUFFIX, "-crumb");
		configuration.add(BreadCrumbsConstants.BREADCRUMBS_DEFAULT_STYLESHEET, "classpath:/org/oscy/tapestry/breadcrumbs/Assets/default.css");
	}

	public static void contributePageRenderRequestHandler(OrderedConfiguration<PageRenderRequestFilter> configuration,final BreadCrumbsService breadCrumbsService) {
		PageRenderRequestFilter breadCrumbsFilter = new PageRenderRequestFilter()
        {
            public void handle(PageRenderRequestParameters parameters, PageRenderRequestHandler handler)
                    throws IOException
            {

				breadCrumbsService.addToBreadCrumbs(parameters);

                handler.handle(parameters);
            }
        };
		configuration.add("zbreadcrumbs", breadCrumbsFilter);
	}

	public void contributeMarkupRenderer(OrderedConfiguration<MarkupRendererFilter> configuration, final Environment environment, @Path("${zbreadcrumbs-default-stylesheet}") final Asset defaultStylesheet) {
		MarkupRendererFilter injectDefaultStylesheet = new MarkupRendererFilter()
		{
			public void renderMarkup(MarkupWriter writer, MarkupRenderer renderer) {
				DocumentLinker linker = environment.peekRequired(DocumentLinker.class);

				linker.addStylesheetLink(new StylesheetLink(defaultStylesheet.toClientURL()));

				renderer.renderMarkup(writer);
			}
		};

		configuration.add("ZBreadCrumbsDefaultStylesheet", injectDefaultStylesheet, "after:InjectDefaultStylesheet");
	}

	public static void contributeApplicationStateManager(MappedConfiguration<Class<?>, ApplicationStateContribution> configuration, @Symbol(BreadCrumbsConstants.BREADCRUMBS_MAX_CRUMBS_TO_SAVE) final int crumbsToSave, @Symbol(BreadCrumbsConstants.BREADCRUMBS_DISCARD_DUPLICATES) final boolean discardDuplicates) {
		ApplicationStateCreator<BreadCrumbsList> breadCrumbsListCreator = new ApplicationStateCreator<BreadCrumbsList>()
		{
			public BreadCrumbsList create() {
				return new BreadCrumbsListImpl(crumbsToSave, discardDuplicates);
			}
		};

		configuration.add(BreadCrumbsList.class, new ApplicationStateContribution("session", breadCrumbsListCreator));
	}

	public static void contributeComponentClassResolver(final Configuration<LibraryMapping> configuration) {
		configuration.add(new LibraryMapping("zcrumb", "org.oscy.tapestry.breadcrumbs"));
	}
}
